"
 COPYRIGHT (c) 1994 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

Block subclass:#CheapBlock
	instanceVariableNames:'selfValue method'
	classVariableNames:''
	poolDictionaries:''
	category:'Kernel-Methods'
!

!CheapBlock class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1994 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    CheapBlocks are blocks which do not need their home-context
    (i.e. blocks that do not access any method arguments 
     or method locals).

    CheapBlocks are never created explicitely; the only creation
    is done by the compilers, when some sourceCode is compiled to either
    machine or byteCode.

    Since they have no reference to the home, they must store their
    creating method explicitely - otherwise, the system had no 
    chance of finding the source-position of the block.

    [Instance variables:]

      selfValue   <Object>          copied self value
                                    (if its a copying block)

      method      <Method>          method where block was created 

    NOTICE: layout known by runtime system and compiler - do not change

    [author:]
        Claus Gittinger

    [see also:]
        Block Context Method
        ( contexts, stacks & unwinding :html: programming/contexts.html )
"
! !

!CheapBlock class methodsFor:'queries'!

isBuiltInClass
    "return true if this class is known by the run-time-system.
     Here, true is returned for myself, false for subclasses."

    ^ self == CheapBlock

    "Created: 16.4.1996 / 11:25:23 / cg"
    "Modified: 23.4.1996 / 15:56:46 / cg"
! !

!CheapBlock methodsFor:'accessing'!

method
    "return the receivers home method.
     Thats the method where the block was created."

    ^ method
!

selfValue
    "return the copied self"

    ^ selfValue
! !

!CheapBlock methodsFor:'printing & storing'!

printOn:aStream
    "append a a printed representation of the block to aStream"

    |class selector pair|

    "
     cheap blocks have no home context, but a method instead
    "

    aStream nextPutAll:'[] in '.
    "
     find out, for which class this method was for ...
    "
    method notNil ifTrue:[
	pair := method who.
	pair notNil ifTrue:[
	    class := pair at:1.
	    selector := pair at:2.
	    aStream nextPutAll:(class name , '-' , selector).
	    aStream nextPutAll:' (optimized)'.
	    ^ self
	].
    ].
    "
     currently, some cheap blocks don't know where they have been created
    "
    aStream nextPutAll:' ??? (optimized)'.
    ^ self
! !

!CheapBlock class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic/CheapBlock.st,v 1.14 1996-04-25 16:49:05 cg Exp $'
! !
